--
-- Washable1315
-- This is the specialization for all washable vehicles
--
-- @author  Manuel Leithner
-- @date  30/07/14
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
--
-- <washable dirtDuration="60" washDuration="1" workMultiplier="10"/>

Washable1315 = {};

function Washable1315.prerequisitesPresent(specializations)
    return true;
end;

function Washable1315:preLoad(xmlFile)
    assert(self.setDirtAmount == nil, "Washable1315 needs to be the first specialization which implements setDirtAmount");
    self.setDirtAmount = Washable1315.setDirtAmount;
    assert(self.getDirtAmount == nil, "Washable1315 needs to be the first specialization which implements getDirtAmount");
    self.getDirtAmount = Washable1315.getDirtAmount;
    assert(self.getDirtMultiplier == nil, "Washable1315 needs to be the first specialization which implements getDirtMultiplier");
	self.getDirtMultiplier = Washable1315.getDirtMultiplier;
end;

function Washable1315:load(xmlFile)

    self.addWashingTrigger = Washable1315.addWashingTrigger;
    self.removeWashingTrigger = Washable1315.removeWashingTrigger;
    self.addWashableNode = Washable1315.addWashableNode;
	self.removeWashableNode = Washable1315.removeWashableNode;
	
    self.sentDirtAmount = 0;	
    self.washableDirtyFlag = self:getNextDirtyFlag();	
    self.dirtAmount = 0;
	
    if hasXMLProperty(xmlFile, "vehicle.dirt#dirtDuration") then

        self.dirtDuration = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dirt#dirtDuration"), 1) * 60 * 1000;
        if self.dirtDuration ~= 0 then
            self.dirtDuration = 1 / self.dirtDuration; -- dirtDuration == 0 disables dirt effect
        end;
        self.washDuration = math.max(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dirt#washDuration"), 1) * 60 * 1000, 0.00001); -- washDuration == 0 washes vehicle immediately
        self.workMultiplier = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dirt#workMultiplier"), 1);
        self.fieldMultiplier = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dirt#fieldMultiplier"), 2);
		
		local i = 0;
		self.dirtComponents = {};
		while true do
			local path = string.format("vehicle.dirt.dirtComponent(%d)", i);
			local component = Utils.indexToObject(self.components, getXMLString(xmlFile, path .. "#index"));	
			if component == nil then
				break;
			end;
			table.insert(self.dirtComponents, component);
			i = i + 1;
		end;	
    end;
	

	
    self.washingTriggers = {};
	
end;


function Washable1315:readStream(streamId, connection)
    if self.dirtComponents ~= nil then
        local dirtAmount = streamReadInt8(streamId) / 100;
        self.sentDirtAmount = dirtAmount;
        self:setDirtAmount(dirtAmount, true);
    end;
end;

function Washable1315:writeStream(streamId, connection)
    if self.dirtComponents ~= nil then
        streamWriteInt8(streamId, math.floor(self:getDirtAmount()*100));
    end;
end;

function Washable1315:readUpdateStream(streamId, timestamp, connection)
    if connection:getIsServer() and self.dirtComponents ~= nil then
        local dirty = streamReadBool(streamId);
        if dirty then
            local dirtAmount = streamReadInt8(streamId) / 100;
            self:setDirtAmount(dirtAmount, true);
        end;
    end;
end;

function Washable1315:writeUpdateStream(streamId, connection, dirtyMask)
    if not connection:getIsServer() and self.dirtComponents ~= nil then
        if streamWriteBool(streamId, bitAND(dirtyMask, self.washableDirtyFlag) ~= 0) then
            streamWriteInt8(streamId, math.floor(self.sentDirtAmount*100));
        end;
    end;
end;

function Washable1315:delete()
    for _, trigger in pairs(self.washingTriggers) do
        trigger:onVehicleDeleted(self);
    end
end;

function Washable1315:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Washable1315:keyEvent(unicode, sym, modifier, isDown)
end;

function Washable1315:update(dt)
end;

function Washable1315:updateTick(dt)
    if self.dirtComponents ~= nil then
        if self.isServer then
            if g_currentMission.environment.lastRainScale > 0.1 and g_currentMission.environment.timeSinceLastRain < 30 then
                local amount = self:getDirtAmount();
                if amount > 0.5 then
                    amount = self:getDirtAmount() - (dt/self.washDuration);
                end;
                self:setDirtAmount(amount);
            else
                if self:getIsActive() or self.isActive then
                    self:setDirtAmount(self:getDirtAmount() + (dt * self.dirtDuration)*self:getDirtMultiplier()*Washable1315.getIntervalMultiplier());
                end;
            end;
        end;
    end;
end;

function Washable1315:draw()
end;

function Washable1315:setDirtAmount(dirtAmount, force)
    if self.dirtComponents ~= nil then
        self.dirtAmount = Utils.clamp(Utils.getNoNil(dirtAmount, self.dirtAmount), 0, 1);
        -- only update shader if dirt change sum is about 1%
        if math.abs(self.sentDirtAmount - self.dirtAmount) > 0.01 or force then
			for _, component in pairs(self.dirtComponents) do
				setShaderParameter(component, "dirtScale", self.dirtAmount, self.dirtAmount, self.dirtAmount, self.dirtAmount, true);
			end;
            if self.isServer then
                -- use min threshold (0.01) for sync limitation
                self:raiseDirtyFlags(self.washableDirtyFlag);
                self.sentDirtAmount = self.dirtAmount;
            end;
        end;
    end;
end;

function Washable1315:addWashableNode(node)
	table.insert(self.dirtComponents, node);
end;

function Washable1315:removeWashableNode(node)
    for i=1, table.getn(self.dirtComponents) do
        if self.dirtComponents[i] == node then
            table.remove(self.dirtComponents, i);
            break;
        end;
    end;
end;

function Washable1315:getDirtAmount(superFunc)
    return self.dirtAmount;
end;

function Washable1315:getDirtMultiplier(superFunc)
    local multiplier = 1;
    if self:getIsOnField() then
        multiplier = multiplier * self.fieldMultiplier;
    end;
    if superFunc ~= nil then
        multiplier = multiplier + superFunc(self);
    end;
    return multiplier;
end;

function Washable1315:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
    if Washable1315.getIntervalMultiplier() ~= 0 then
        self:setDirtAmount(Utils.getNoNil(getXMLFloat(xmlFile, key.."#dirtAmount"), 0), true);
    end
    return BaseMission.VEHICLE_LOAD_OK;
end;

function Washable1315:getSaveAttributesAndNodes(nodeIdent)
    local attributes = 'dirtAmount="'.. tostring(self.dirtAmount) .. '"';
    return attributes, nil;
end;

function Washable1315:addWashingTrigger(trigger)
    table.insert(self.washingTriggers, trigger);
end;

function Washable1315:removeWashingTrigger(trigger)
    for i=1, table.getn(self.washingTriggers) do
        if self.washingTriggers[i] == trigger then
            table.remove(self.washingTriggers, i);
            break;
        end;
    end;
end;

function Washable1315.getIntervalMultiplier()
    if g_settingsDirtInterval == 1 then
        return 0;
    elseif g_settingsDirtInterval == 2 then
        return 0.25;
    elseif g_settingsDirtInterval == 3 then
        return 0.5;
    elseif g_settingsDirtInterval == 4 then
        return 1;
    end;
end;
